/*****************************************/
/*****************************************/
/**              CLAUSES                **/
/**                                     **/
/** Managing and solving a set of       **/
/** clauses                             **/
/**                                     **/
/** Copyright (c) 2008                  **/
/**     Stefan Stattelmann              **/
/**                                     **/
/** Author:                             **/
/**     Stefan Stattelmann              **/
/**                                     **/
/** Contact:                            **/
/**     s9ststat@stud.uni-saarland.de   **/
/*****************************************/
/*****************************************/
#ifndef _CLAUSES_
#define _CLAUSES_

/************/
/* Includes */
/************/
#include <string.h>
#include "list.h"
#include "parse.h"

/**************/
/* Structures */
/**************/

typedef enum VAR_STATE_HELP {
    VAR_UNDEFINED = -1,
    VAR_FALSE = 0,
    VAR_TRUE = 1
} VAR_STATE;

/**************************************************************/
/**************************************************************/
/*  This enum type is used to encode the value of a truth     */
/*  variable in a valuation                                   */
/**************************************************************/


typedef struct CLAUSES_HELP {
    int num_vars;
    int num_clauses;
    LIST* clauses;
    VAR_STATE* valuation;
    int* backtrack;
    int back_pos;
    int* dep;
    int* ranking;
    int rank_pos;
} CLAUSES_NODE;

typedef CLAUSES_NODE *CLAUSES;

/**************************************************************/
/**************************************************************/
/*  This is the structure to implement a set of clauses and   */
/*  to find a valuation that makes all clauses true           */
/*      num_vars    number of variables used                  */
/*      num_clauses number of clauses in the clause set       */
/*      clauses     an array of size num_clauses of lists of  */
/*                  literals (which are encoded as signed     */
/*                  integers)                                 */
/*      valuation   array storing truth values of variables   */
/*      backtrack   backtrack stack, implemented as array of  */
/*                  size num_vars                             */
/*      back_pos    next free position in backtrack array     */
/*      dep         array storing variable/clause dependencies*/
/*      ranking     ranking of most used variables            */
/*      rank_pos    next position to select from the ranking  */
/**************************************************************/
/**************************************************************/


/**************/
/* Functions  */
/**************/

void    clauses_Init (void);
void    clauses_Free (void);

CLAUSES clauses_Parse (const char*);
void clauses_Delete (CLAUSES);
int clauses_Solve (CLAUSES);
void clauses_PrintValuation (CLAUSES, FILE*);

#endif

