/*****************************************/
/*****************************************/
/**               LIST                  **/
/**                                     **/
/** Generic list implementation         **/
/**                                     **/
/** Copyright (c) 2008                  **/
/**     Stefan Stattelmann              **/
/**                                     **/
/** Author:                             **/
/**     Stefan Stattelmann              **/
/**                                     **/
/** Contact:                            **/
/**     s9ststat@stud.uni-saarland.de   **/
/*****************************************/
/*****************************************/

#include "list.h"

/**************/
/* Functions  */
/**************/

void list_Init (void)
/**************************************************************
 INPUT:     nothing
 RETURNS:   void
 SUMMARY:   must be here
***************************************************************/
{
}

void list_Free (void)
/**************************************************************
 INPUT:     nothing
 RETURNS:   void
 SUMMARY:   must be here
***************************************************************/
{
}

LIST list_Create  (void *d)
/**************************************************************
 INPUT:     data element
 RETURNS:   list with one element
 SUMMARY:   create an one element list
***************************************************************/
{
    return list_Prepend (d, NULL);
}

LIST list_Prepend (void *d, LIST l)
/**************************************************************
 INPUT:     data element, list
 RETURNS:   list with an additional element
 SUMMARY:   add a given element to the front of a list
***************************************************************/
{
    LIST h;

    h = (LIST) malloc (sizeof (LIST_NODE));

    if (h)
    {
        h->data = d;
        h->tail = l;
    }

    return h;
}

void* list_Head (LIST l)
/**************************************************************
 INPUT:     list
 RETURNS:   data stored in first element
 SUMMARY:   retrieve first element of a list without changing
            the list
***************************************************************/
{
    if (l)
        return l->data;

    return NULL;
}

LIST list_Tail (LIST l)
/**************************************************************
 INPUT:     list
 RETURNS:   list
 SUMMARY:   throw away first element of a list
***************************************************************/
{
    if (l)
        return l->tail;

    return NULL;
}

int list_Nil (LIST l)
/**************************************************************
 INPUT:     list
 RETURNS:   zero if list is not NULL
 SUMMARY:   test if a list is empty
***************************************************************/
{
    return !l;
}

void list_NodeFree (LIST l, int flag)
/**************************************************************
 INPUT:     list, flag for what should be freed
 RETURNS:   void
 SUMMARY:   Free memory for this node of the list but not the 
            rest of the list. If the flag is nonzero free data
            element, too.
***************************************************************/
{
    if (l && flag)
        free (l->data);

    free (l);
}

void list_Delete (LIST l, int flag)
/**************************************************************
 INPUT:     list, flag for what should be freed
 RETURNS:   void
 SUMMARY:   Free memory of this list. If the flag is nonzero
            free data element, too.
***************************************************************/
{
    if (l)
    {
        list_Delete (l->tail, flag);
        list_NodeFree (l, flag);
    }
}


LIST list_Concat (LIST l1, LIST l2)
/**************************************************************
 INPUT:     two lists
 RETURNS:   concatenation of the lists
 MEMORY:    no new memory is allocated
 CAUTION:   very inefficient
 SUMMARY:   Free memory of this list. If the flag is nonzero
            free data element, too.
***************************************************************/
{
    LIST it;

    if (l1 == NULL)
        return l2;
    if (l2 == NULL)
        return l1;

    it = l1;

    while (it->tail != NULL)
        it = it->tail;

    it->tail = l2;

    return l1;
}
